<?php

/**
 * Load Backend related actions
 *
 * @class   ACOOFM_Backend
 */

if (!defined('ABSPATH')) {
    exit;
}


class ACOOFM_Backend
{


    /**
     * Class intance for singleton  class
     *
     * @var    object
     * @access  private
     * @since    1.0.0
     */
    private static $instance = null;

    /**
     * The version number.
     *
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $version;

    /**
     * The token.
     *
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $token;

    /**
     * The main plugin file.
     *
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $file;

    /**
     * The main plugin directory.
     *
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $dir;

    /**
     * The plugin assets directory.
     *
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_dir;

    /**
     * Suffix for Javascripts.
     *
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $script_suffix;

    /**
     * The plugin assets URL.
     *
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_url;
    /**
     * The plugin hook suffix.
     *
     * @var     array
     * @access  public
     * @since   1.0.0
     */
    public $hook_suffix = array();


    /**
     * Constructor function.
     *
     * @access  public
     * @param string $file plugin start file path.
     * @since   1.0.0
     */
    public function __construct($file = '')
    {
        $this->version = ACOOFM_VERSION;
        $this->token = ACOOFM_TOKEN;
        $this->file = $file;
        $this->dir = dirname($this->file);
        $this->assets_dir = trailingslashit($this->dir) . 'assets';
        $this->assets_url = esc_url(trailingslashit(plugins_url('/assets/', $this->file)));
        $this->script_suffix = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ? '' : '.min';
        $plugin = plugin_basename($this->file);

        // add action links to link to link list display on the plugins page.
        add_filter("plugin_action_links_$plugin", array($this, 'pluginActionLinks'));

        // reg activation hook.
        register_activation_hook($this->file, array($this, 'install'));
        // reg deactivation hook.
        register_deactivation_hook($this->file, array($this, 'deactivation'));

        // Admin Init
        add_action('admin_init', array($this, 'adminInit'));

        // reg admin menu.
        add_action('admin_menu', array($this, 'registerRootPage'), 30);

        // Init functions, you can use it for posttype registration and all.

        // enqueue scripts & styles.
        add_action('admin_enqueue_scripts', array($this, 'adminEnqueueScripts'), 10, 1);
        add_action('admin_enqueue_scripts', array($this, 'adminEnqueueStyles'), 10, 1);
    }


    /**
     * Method that is used on plugin initialization time
     * @since 1.0.0
     */
    public function adminInit() {
        if(get_option( $this->token.'_db_version' ) !== ACOOFM_DB_VERSION) {
            $this->do_database_upgrade();
        }

        if (get_option($this->token.'_do_activation_redirect', false)) {
            delete_option($this->token.'_do_activation_redirect');
            wp_redirect(admin_url('admin.php?page=' . $this->token . '-admin-ui#/configure'));
        }
    }

    /**
     * Database Upgrade 
     * @since 1.0.1
     */
    private function do_database_upgrade() {
        global $wpdb;
        $table_name         = ACOOFM_ITEM_TABLE;
        $current_version    = get_option( $this->token.'_db_version' );

        switch($current_version) {
            case '1.0.0':
                $wpdb->query( "ALTER TABLE $table_name DROP INDEX uidx_source_url;");
                $wpdb->query( "ALTER TABLE $table_name DROP source_url;");
                
                $files = $wpdb->get_results( "SELECT * FROM $table_name;");
                if($files) {
                    foreach($files as $file) {
                        $rel_path   = acoofm_get_attachment_relative_path($file->source_path);

                        // Do multiple iteration to correct url
                        for($i = 0; $i<4; $i++) {
                            acoofm_get_attachment_relative_path($rel_path);
                        }

                        $id         = $file->id;
                        $wpdb->query( "UPDATE $table_name SET source_path = '$rel_path' WHERE id = $id;");
                    }
                }
                break;
            default: null;
        }

        update_option( $this->token.'_db_version', ACOOFM_DB_VERSION, true );
    }
    
    /**
     * Ensures only one instance of Class is loaded or can be loaded.
     *
     * @param string $file plugin start file path.
     * @return Main Class instance
     * @since 1.0.0
     * @static
     */
    public static function instance($file = '')
    {
        if (is_null(self::$instance)) {
            self::$instance = new self($file);
        }
        return self::$instance;
    }


    /**
     * Show action links on the plugin screen.
     *
     * @param mixed $links Plugin Action links.
     *
     * @return array
     */
    public function pluginActionLinks($links)
    {
        $action_links = array(
            'getstarted' => '<a href="' . admin_url('admin.php?page=' . $this->token . '-admin-ui#/configure') . '">' . esc_html__('Get Started', 'offload-media-cloud-storage') . '</a>',
            'settings' => '<a href="' . admin_url('admin.php?page=' . $this->token . '-admin-ui/') . '">' . esc_html__('Settings', 'offload-media-cloud-storage') . '</a>',
        );

        return array_merge($action_links, $links);
    }


    /**
     * Installation. Runs on activation.
     *
     * @access  public
     * @return  void
     * @since   1.0.0
     */
    public function install()
    {
        global $wpdb;

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

        $table_name = ACOOFM_ITEM_TABLE;
        $queries = array();
        $charset_collate = $wpdb->get_charset_collate();
        if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) {
            $queries[] = "
                CREATE TABLE {$table_name} (
                id BIGINT(20) NOT NULL AUTO_INCREMENT,
                provider varchar(18) NOT NULL,
                region varchar(255) NOT NULL,
                bucket varchar(255) NOT NULL,
                source_id bigint(20) NOT NULL,
                source_path varchar(1024) NOT NULL,
                source_type varchar(18) NOT NULL,
                url varchar(1024) NOT NULL,
                path varchar(1024) NOT NULL,
                is_private tinyint(1) NOT NULL DEFAULT 0,
                extra_info longtext DEFAULT NULL,
                PRIMARY KEY (id),
                UNIQUE KEY uidx_url (url(190), id),
                UNIQUE KEY uidx_path (path(190), id),
                UNIQUE KEY uidx_source_path (source_path(190), id)
            ) $charset_collate;
            ";
        } 

        dbDelta( $queries );

        // Redirection on activation
        add_option($this->token.'_do_activation_redirect', true);

        //Protect directories
        $this->_protect_upload_dir();
    }

    /**
     * Protect Directory from external access.
     *
     * @access  private
     * @return  void
     * @since   1.0.0
     */
    private function _protect_upload_dir(){
        $upload_dir = wp_upload_dir();

        $files = array(
            array(
                'base' => $upload_dir['basedir'] . '/' . ACOOFM_UPLOADS,
                'file' => '.htaccess',
                'content' => 'Options -Indexes' . "\n"
                    . '<Files *.php>' . "\n"
                    . 'deny from all' . "\n"
                    . '</Files>'
            ),
            array(
                'base' => $upload_dir['basedir'] . '/' . ACOOFM_UPLOADS,
                'file' => 'index.php',
                'content' => '<?php ' . "\n"
                    . '// Silence is golden.'
            )
        );

        foreach ($files as $file) {


            if ((wp_mkdir_p($file['base'])) && (!file_exists(trailingslashit($file['base']) . $file['file']))  // If file not exist
            ) {
                if ($file_handle = @fopen(trailingslashit($file['base']) . $file['file'], 'w')) {
                    fwrite($file_handle, $file['content']);
                    fclose($file_handle);
                }
            }
        }
    }


   
    /**
     * Creating admin pages
     */
    public function registerRootPage()
    {
        $this->hook_suffix[] = add_menu_page( 
            __('Offload Media', 'offload-media-cloud-storage'),
            __('Offload Media', 'offload-media-cloud-storage'),
            'manage_options', 
            $this->token . '-admin-ui', 
            array($this, 'adminUi'), 
            esc_url($this->assets_url) . '/images/acoofm-logo.svg', 25
        );
        $this->hook_suffix[] = add_submenu_page(
            $this->token . '-admin-ui', 
            __('Dashboard', 'offload-media-cloud-storage'), 
            __('Dashboard', 'offload-media-cloud-storage'), 
            'manage_options', 
            $this->token . '-admin-ui' ,
            array($this, 'adminUi')
        );

        if(acoofm_is_service_enabled()) {
            $this->hook_suffix[] = add_submenu_page(
                $this->token . '-admin-ui', 
                __('Settings', 'offload-media-cloud-storage'), 
                __('Settings', 'offload-media-cloud-storage'), 
                'manage_options', 
                $this->token . '-admin-ui#/settings' ,
                array($this, 'adminUi')
            );
        }

        $this->hook_suffix[] = add_submenu_page(
            $this->token . '-admin-ui', 
            __('Configure', 'offload-media-cloud-storage'), 
            __('Configure', 'offload-media-cloud-storage'), 
            'manage_options', 
            $this->token . '-admin-ui#/configure' ,
            array($this, 'adminUi')
        );
        // $this->hook_suffix[] = add_submenu_page(
        //     $this->token . '-admin-ui', 
        //     __('License', 'offload-media-cloud-storage'), 
        //     __('License', 'offload-media-cloud-storage'), 
        //     'manage_options', 
        //     $this->token . '-admin-ui#/license' ,
        //     array($this, 'adminUi')
        // );
        $this->hook_suffix[] = add_submenu_page(
            $this->token . '-admin-ui', 
            __('Documentation', 'offload-media-cloud-storage'), 
            __('Documentaion', 'offload-media-cloud-storage'), 
            'manage_options', 
            'https://acowebs.com'
        );
    }

    /**
     * Calling view function for admin page components
     */
    public function adminUi()
    {

        echo (
            '<div id="' . $this->token . '_ui_root">
            <div class="' . $this->token . '_loader"><h1>' . __('Offload Media - Cloud Storage', 'offload-media-cloud-storage') . '</h1><p>' . __('Plugin is loading Please wait for a while..', 'offload-media-cloud-storage') . '</p></div>
            </div>'
        );
    }

    /**
     * Load admin CSS.
     *
     * @access  public
     * @return  void
     * @since   1.0.0
     */
    public function adminEnqueueStyles()
    {
        wp_enqueue_style( 'wpb-google-fonts-dm-sans', 'https://fonts.googleapis.com/css2?family=DM+Sans:ital,wght@0,400;0,500;0,700;1,400;1,500;1,700', false );
        wp_enqueue_style( 'wpb-google-fonts-roboto', 'https://fonts.googleapis.com/css2?family=Roboto:wght@400;500;700', false );
        wp_register_style($this->token . '-admin', esc_url($this->assets_url) . 'css/backend.css', array(), $this->version);
        wp_enqueue_style($this->token . '-admin');
    }

    /**
     * Load admin Javascript.
     *
     * @access  public
     * @return  void
     * @since   1.0.0
     */
    public function adminEnqueueScripts()
    {
        if (!isset($this->hook_suffix) || empty($this->hook_suffix)) {
            return;
        }

        $screen = get_current_screen();

        if (in_array($screen->id, $this->hook_suffix, true)) {
            // Enqueue WordPress media scripts.
            if (!did_action('wp_enqueue_media')) {
                wp_enqueue_media();
            }

            if (!wp_script_is('wp-i18n', 'registered')) {
                wp_register_script('wp-i18n', esc_url($this->assets_url) . 'js/i18n.min.js', array(), $this->version, true);
            }
            // Enqueue custom backend script.
            wp_enqueue_script($this->token . '-backend', esc_url($this->assets_url) . 'js/backend.js', array('wp-i18n'), $this->version, true);


            
            // Localize a script.
            wp_localize_script(
                $this->token . '-backend',
                $this->token . '_object',
                array(
                    'api_nonce' => wp_create_nonce('wp_rest'),
                    'root' => rest_url($this->token . '/v1/'),
                    'assets_url' => $this->assets_url,
                )
            );
        }
        wp_enqueue_script($this->token . '-media', esc_url($this->assets_url) . 'js/media.js', array('jquery'), $this->version, true);
        // Localize a script.
        wp_localize_script(
            $this->token . '-media',
            $this->token . '_media_object',
            array(
                'file_details_nonce'    => wp_create_nonce('get_media_provider_details'),
                'admin_ajax_url'        => admin_url('admin-ajax.php'),
                'strings'               => array(
                    'provider'          => __("Provider: ", "offload-media-cloud-storage"),
                    'region'            => __("Region: ", "offload-media-cloud-storage")
                )      
            )
        );
    }


    /**
     * Deactivation hook
     */
    public function deactivation()
    {
    }

    /**
     * Cloning is forbidden.
     *
     * @since 1.0.0
     */
    public function __clone()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->_version);
    }

    /**
     * Unserializing instances of this class is forbidden.
     *
     * @since 1.0.0
     */
    public function __wakeup()
    {
        _doing_it_wrong(__FUNCTION__, __('Cheatin&#8217; huh?'), $this->_version);
    }
}
